/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2016-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */
import { SPainter, SPoint, SPath } from "@persagy-web/draw";
import { SLineStyle, SGraphItem } from "@persagy-web/graph/";
import { SBaseRectEdit } from "..";
import { Marker } from "@persagy-web/big/lib";
import { SItemStatus } from "@persagy-web/big/lib";
import { SMouseEvent } from "@persagy-web/base";

/**
 * 矩形编辑类
 *
 * @author 郝建龙 <haojianlong@sagacloud.cn>
 */
export default class SBaseMaskEdit extends SBaseRectEdit {
    /** 绘制的路径 */
    mask: SPath | undefined;

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      矩形数据
     */
    constructor(parent: SGraphItem | null, data: Marker) {
        super(parent, data);
        this.data = data;
    }

    /**
     * 计算矩形的左上角和右下角
     */
    protected calRect(): void {
        // 顶点个数超过1个
        if (this.line.length > 1) {
            const fi = this.line[0];
            const se = this.line[1];
            let minx, maxx, miny, maxy;
            // 比较并获得最大 x 值和最小 x 值
            // 第一个点的 x 坐标较小
            if (fi.x < se.x) {
                minx = fi.x;
                maxx = se.x;
            } else {
                // 否则
                minx = se.x;
                maxx = fi.x;
            }

            // 比较并获得最大 y 值和最小 y 值
            // 第一个坐标的 y 坐标较小
            if (fi.y < se.y) {
                miny = fi.y;
                maxy = se.y;
            } else {
                miny = se.y;
                maxy = fi.y;
            }

            this._leftTop = new SPoint(minx, miny);
            this._rightBottom = new SPoint(maxx, maxy);
            this.createMask();
        }
    }

    /**
     * 创建蒙版
     */
    createMask(): void {
        this.mask = new SPath();
        this.mask.moveTo(Number.MIN_SAFE_INTEGER, Number.MIN_SAFE_INTEGER);
        this.mask.lineTo(Number.MAX_SAFE_INTEGER, Number.MIN_SAFE_INTEGER);
        this.mask.lineTo(Number.MAX_SAFE_INTEGER, Number.MAX_SAFE_INTEGER);
        this.mask.lineTo(Number.MIN_SAFE_INTEGER, Number.MAX_SAFE_INTEGER);
        this.mask.lineTo(Number.MIN_SAFE_INTEGER, Number.MIN_SAFE_INTEGER);

        let antiArr = [
            this._leftTop,
            new SPoint(this._leftTop.x, this._rightBottom.y),
            this._rightBottom,
            new SPoint(this._rightBottom.x, this._leftTop.y)
        ];
        this.mask.polygon(antiArr);
        this.update();
    }

    /**
     * 是否包含点
     *
     * @param x 横坐标
     * @param y 纵坐标
     * @return 是否包含
     */
    contains(x: number, y: number): boolean {
        return !this.boundingRect().contains(x, y);
    }

    onMouseUp(event: SMouseEvent): boolean {
        // 处于非创建态
        if (this.status != SItemStatus.Create) {
            // 被选中
            if (this.selected) {
                try {
                    // @ts-ignore
                    this.parent.scene.selectContainer.clear();
                } catch (e) {
                    console.log(e);
                }
            } else {
                super.onMouseUp(event);
            }
        }
        return true;
    }

    /**
     * Item 绘制操作
     *
     * @param painter   绘制对象
     */
    onDraw(painter: SPainter): void {
        // 顶点个数为 2 个
        if (this.line.length == 2) {
            painter.pen.color = this.strokeColor;
            painter.brush.color = this.fillColor;
            const eachPx = painter.toPx(1);
            // 使用像素值
            if (this.widthIsPx) {
                painter.pen.lineWidth = this.lineWidth * eachPx;
            } else {
                // 否则
                painter.pen.lineWidth = this.lineWidth;
            }
            // 虚线类型
            if (this.lineStyle == SLineStyle.Dashed) {
                painter.pen.lineDash = [eachPx * 3, eachPx * 7];
            } else if (this.lineStyle == SLineStyle.Dotted) {
                // 点线类型
                painter.pen.lineDash = [2 * eachPx, 2 * eachPx];
            }

            // 蒙版生成
            if (this.mask) {
                painter.drawPath(this.mask);
            }
        }
    }
}
