/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import {
    SGraphItem,
    SGraphPointListDelete,
    SGraphPointListInsert,
    SGraphPointListUpdate,
    SLineStyle
} from "@persagy-web/graph";
import { SKeyCode, SMouseEvent, SUndoStack } from "@persagy-web/base/";
import {
    SColor,
    SLine,
    SLineCapStyle,
    SPainter,
    SPoint,
    SPolygonUtil,
    SRect,
    SSize
} from "@persagy-web/draw";
import { SItemStatus, ItemOrder } from "@persagy-web/big";
import { Marker } from "@persagy-web/big/lib";
import { SMathUtil } from "@persagy-web/big/lib/utils/SMathUtil";
import { SGraphEdit } from "..";

/**
 * 编辑多边形
 *
 * @author 韩耀龙 <han_yao_long@163.com>
 */
export class SBasePolygonEdit extends SGraphEdit {
    /** X 坐标最小值 */
    private minX = Number.MAX_SAFE_INTEGER;
    /** X 坐标最大值 */
    private maxX = Number.MIN_SAFE_INTEGER;
    /** Y 坐标最小值 */
    private minY = Number.MAX_SAFE_INTEGER;
    /** Y 坐标最大值 */
    private maxY = Number.MIN_SAFE_INTEGER;
    /** 图例节点对象数据 */
    data: Marker;
    /** 轮廓线坐标 */
    private pointList: SPoint[] = [];
    get getPointList(): SPoint[] {
        return this.pointList;
    }
    set setPointList(arr: SPoint[]) {
        this.pointList = arr;
        this.update();
    }

    /** 当前状态 */
    protected _status: number = SItemStatus.Normal;
    get status(): SItemStatus {
        return this._status;
    }
    set status(value: SItemStatus) {
        this._status = value;
        this.undoStack.clear();
        this.update();
    }

    /** 边框颜色 */
    _strokeColor: SColor = SColor.Black;
    /** 画笔颜色 */
    get strokeColor(): SColor {
        return this._strokeColor;
    }
    set strokeColor(v: SColor) {
        this._strokeColor = v;
        this.update();
    }

    /** 填充颜色 */
    _fillColor: SColor = new SColor("#2196f3");
    get fillColor(): SColor {
        return this._fillColor;
    }
    set fillColor(v: SColor) {
        this._fillColor = v;
        this.update();
    }

    /** 边框样式 */
    _lineStyle: SLineStyle = SLineStyle.Solid;
    get lineStyle(): SLineStyle {
        return this._lineStyle;
    }
    set lineStyle(v: SLineStyle) {
        this._lineStyle = v;
        this.update();
    }

    /** 边框的宽只可输入像素宽 */
    _lineWidth: number = 1;
    get lineWidth(): number {
        return this._lineWidth;
    }
    set lineWidth(v: number) {
        this._lineWidth = v;
        this.update();
    }

    /** 是否闭合 */
    closeFlag: boolean = false;
    /** 鼠标移动点 */
    private lastPoint: SPoint | null = null;
    /** 当前鼠标获取顶点对应索引 */
    private curIndex: number = -1;
    /** 当前鼠标获取顶点对应坐标 */
    private curPoint: null | SPoint = null;
    /** 灵敏像素 */
    private len: number = 10;
    /** 场景像素内部将灵敏像素换算为场景实际距离 */
    private scenceLen: number = 15;
    /** 场景像素 */
    private isAlt: boolean = false;
    /** undo/redo 堆栈 */
    protected undoStack: SUndoStack = new SUndoStack();

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      图例节点对象数据
     */
    constructor(parent: SGraphItem | null, data: Marker) {
        super(parent);
        // this.textItem.originPosition = STextOrigin.Centrum;
        // this.textItem.isTransform = false;
        this.zOrder = ItemOrder.polygonOrder;
        this.data = data;
        this.name = data.name;
        this.showSelect = false;
        // this.text = data.Name;
        // 样式存在
        if (data.style) {
            this.setPointList = [];
            let setPointList: SPoint[];
            // 样式中存有轮廓线数据
            if (data.style.outLine) {
                // 轮廓线内容类型是 SPoint 类型
                if (data.style.outLine[0] instanceof SPoint) {
                    // @ts-ignore
                    this.setPointList = data.style.outLine;
                } else {
                    // 否则
                    setPointList = data.style.outLine.map(i => {
                        return new SPoint(i.x, i.y);
                    });
                    this.setPointList = setPointList;
                }
            }

            // 样式相关
            if (data.style.default) {
                // 线条颜色
                if (data.style.default.strokeColor) {
                    this.strokeColor = new SColor(
                        data.style.default.strokeColor
                    );
                }

                // 线宽
                if (data.style.default.lineWidth) {
                    this.lineWidth = data.style.default.lineWidth;
                }

                // 线样式
                if (data.style.default.lineStyle) {
                    this.lineStyle = data.style.default.lineStyle;
                }

                // 填充色
                if (data.style.default.fillColor) {
                    this.fillColor = new SColor(data.style.default.fillColor);
                }
            }
        }
        // if (data) {
        //     if (data.Properties.Zorder) {
        //         this.zOrder = data.Properties.Zorder;
        //     }
        //     // 设置线宽

        //     if (data.Properties.TextPos) {
        //         this.textItem.moveTo(data.Properties.TextPos.X, data.Properties.TextPos.Y);
        //     }
        //     if (data.Properties.color) {
        //         this.color = new SColor(data.Properties.color);
        //     }
        //     if (data.Properties.font) {
        //         this.font = new SFont("sans-serif", data.Properties.font);
        //     }
        //     if (data.Properties && data.Properties.IsActive) {
        //         this.isActive = data.Properties.IsActive;
        //     }
        // }
        // 监听多边形创建完成事件,并动态计算文本位置
        // this.connect("finishCreated", this, () => {
        //     // 计算文本位置
        //     let x: number = this.getPointList.reduce((pre, cur, index, arr) => {
        //         return pre + (cur.x / arr.length)
        //     }, 0),
        //         y: number = this.getPointList.reduce((pre, cur, index, arr) => {
        //             return pre + (cur.y / arr.length)
        //         }, 0);
        //     this.textItem.moveTo(x, y);
        // })
    }

    //////////////////
    //  以下为对pointList 数组的操作方法

    /**
     * 储存新的多边形顶点
     *
     * @param x   点位得 x 坐标
     * @param y   点位得 y 坐标
     * @param i   储存所在索引
     * @return 添加的顶点
     */
    insertPoint(x: number, y: number, i: number | null = null): SPoint {
        const point = new SPoint(x, y);
        // 没有传入索引
        if (i == null) {
            this.pointList.push(point);
        } else {
            // 否则
            this.pointList.splice(i, 0, point);
        }

        this.update();
        return point;
    }

    /**
     * 删除点位
     *
     * @param i   删除点所在的索引
     * @return   索引不在数组范围则返回 null
     */
    deletePoint(i: number | null = null): SPoint | null {
        let point;
        // 传入删除的索引
        if (i != null) {
            // 索引值无效
            if (i >= this.pointList.length || i < 0) {
                point = null;
            } else {
                // 否则
                point = new SPoint(this.pointList[i].x, this.pointList[i].y);
                this.pointList.splice(i, 1);
            }
        } else {
            // 否则
            // 轮廓线数据存在
            if (this.pointList.length) {
                point = this.pointList[this.pointList.length - 1];
                this.pointList.pop();
            } else {
                // 否则
                point = null;
            }
        }

        this.curIndex = -1;
        this.curPoint = null;
        this.update();
        return point;
    }

    /**
     * 多边形顶点的移动位置
     *
     * @param x   点位得 x 坐标
     * @param y   点位得 y 坐标
     * @param i   点位得 i 坐标
     * @return    移动对应得点。如果索引无法找到移动顶点，则返回 null
     */
    movePoint(x: number, y: number, i: number): SPoint | null {
        let point;
        // 索引值无效
        if (i >= this.pointList.length || i < 0) {
            return null;
        }

        // 轮廓线数据存在
        if (this.pointList.length) {
            this.pointList[i].x = x;
            this.pointList[i].y = y;
        }

        point = this.pointList[i];
        return point;
    }

    /**
     * 展示状态 -- 绘制多边形数组
     *
     * @param painter      绘制类
     * @param pointList    绘制多边形数组
     */
    protected drawShowPolygon(painter: SPainter, pointList: SPoint[]): void {
        painter.save();
        painter.pen.lineCapStyle = SLineCapStyle.Square;
        painter.pen.color = this.strokeColor;
        painter.brush.color = this.fillColor;
        painter.pen.lineWidth = painter.toPx(this.lineWidth);
        // 线类型为虚线
        if (this.lineStyle == SLineStyle.Dashed) {
            painter.pen.lineDash = [
                painter.toPx(this.lineWidth * 3),
                painter.toPx(this.lineWidth * 7)
            ];
        } else if (this.lineStyle == SLineStyle.Dotted) {
            // 线类型为点线
            painter.pen.lineDash = [
                painter.toPx(this.lineWidth),
                painter.toPx(this.lineWidth)
            ];
        }

        // 被选中
        if (this.selected) {
            painter.shadow.shadowBlur = 10;
            painter.shadow.shadowColor = new SColor(`#00000033`);
            painter.shadow.shadowOffsetX = 5;
            painter.shadow.shadowOffsetY = 5;
        } else {
            // 否则
            painter.shadow.shadowColor = SColor.Transparent;
        }

        painter.drawPolygon([...pointList]);
        painter.restore();
    }

    /**
     * 创建状态 -- 绘制多边形数组
     *
     * @param painter      绘制类
     * @param pointList    绘制多边形数组
     */
    protected drawCreatePolygon(painter: SPainter, pointList: SPoint[]): void {
        painter.pen.lineCapStyle = SLineCapStyle.Square;
        painter.pen.color = this.strokeColor;
        painter.pen.lineWidth = painter.toPx(this.lineWidth);
        // 最后一个点存在而且轮廓线数据有效
        if (this.lastPoint && pointList.length) {
            painter.drawLine(
                pointList[pointList.length - 1].x,
                pointList[pointList.length - 1].y,
                this.lastPoint.x,
                this.lastPoint.y
            );
        }

        painter.drawPolyline(pointList);
        painter.pen.color = SColor.Transparent;
        painter.brush.color = new SColor(this.fillColor.value);
        painter.pen.lineWidth = painter.toPx(this.lineWidth);
        // 最后一个点存在
        if (this.lastPoint) {
            painter.drawPolygon([...pointList, this.lastPoint]);
            // 绘制顶点块
            painter.pen.color = SColor.Black;
            painter.brush.color = SColor.White;
            pointList.forEach(item => {
                painter.drawCircle(item.x, item.y, painter.toPx(this.len / 2));
            });
            // 轮廓线有效
            if (this.pointList.length) {
                // 如果最后一个点在第一个点的灵敏度范围内,第一个点填充变红
                if (
                    SMathUtil.pointDistance(
                        this.lastPoint.x,
                        this.lastPoint.y,
                        this.pointList[0].x,
                        this.pointList[0].y
                    ) < this.scenceLen
                ) {
                    // 绘制第一个点的顶点块
                    painter.pen.color = SColor.Black;
                    painter.brush.color = SColor.Red;
                    painter.drawCircle(
                        this.pointList[0].x,
                        this.pointList[0].y,
                        painter.toPx(this.len / 2)
                    );
                }
            }
        } else {
            // 否则
            painter.drawPolygon(pointList);
        }
    }

    /**
     *
     * 编辑状态 -- 绘制多边形数组
     *
     * @param painter      绘制类
     * @param pointList    绘制多边形数组
     */
    protected drawEditPolygon(painter: SPainter, pointList: SPoint[]): void {
        // 展示多边形
        painter.pen.lineCapStyle = SLineCapStyle.Square;
        painter.pen.color = this.strokeColor;
        painter.pen.lineWidth = painter.toPx(this.lineWidth);
        painter.brush.color = new SColor(this.fillColor.value);
        painter.drawPolygon([...pointList]);
        // 绘制顶点块
        painter.pen.color = SColor.Black;
        painter.brush.color = SColor.White;
        // 遍历轮廓线列表
        pointList.forEach((item, index) => {
            painter.brush.color = SColor.White;
            // 设置选中点样式
            if (index == this.curIndex) {
                painter.brush.color = new SColor("#2196f3");
            }

            painter.drawCircle(item.x, item.y, painter.toPx(this.len / 2));
        });
    }

    /**
     * 编辑状态操作多边形数组
     *
     * @param event    鼠标事件
     */
    protected editPolygonPoint(event: SMouseEvent): void {
        //  判断是否为删除状态 isAlt = true为删除状态
        if (this.isAlt) {
            // 1 判断是否点击在多边形顶点
            let lenIndex = -1; // 当前点击到的点位索引；
            let curenLen = this.scenceLen; // 当前的灵敏度
            // 遍历轮廓线列表
            this.pointList.forEach((item, index) => {
                let dis = SMathUtil.pointDistance(
                    event.x,
                    event.y,
                    item.x,
                    item.y
                );
                // 鼠标点在线段的吸附范围内
                if (dis < curenLen) {
                    curenLen = dis;
                    lenIndex = index;
                }
            });

            // 若点击到，对该索引对应的点做删除
            if (lenIndex != -1) {
                // 顶点个数小于3个
                if (this.pointList.length <= 3) {
                    return;
                }

                const delePoint = new SPoint(
                    this.pointList[lenIndex].x,
                    this.pointList[lenIndex].y
                );
                this.deletePoint(lenIndex);
                // 记录顶点操作记录压入堆栈
                this.recordAction(SGraphPointListDelete, [
                    this.pointList,
                    delePoint,
                    lenIndex
                ]);
            }
        } else {
            // 否则
            // 1 判断是否点击在多边形顶点
            this.curIndex = -1;
            this.curPoint = null;
            let lenIndex = -1; // 当前点击到的点位索引；
            let curenLen = this.scenceLen; // 当前的灵敏度
            // 遍历轮廓线数据
            this.pointList.forEach((item, index) => {
                let dis = SMathUtil.pointDistance(
                    event.x,
                    event.y,
                    item.x,
                    item.y
                );
                // 鼠标点在线段的吸附范围内
                if (dis < curenLen) {
                    curenLen = dis;
                    lenIndex = index;
                }
            });
            this.curIndex = lenIndex;
            // 有选中的点
            // 2判断是否点击在多边形得边上
            if (-1 == lenIndex) {
                let len = SMathUtil.pointToLine(
                        new SPoint(event.x, event.y),
                        new SLine(this.pointList[0], this.pointList[1])
                    ),
                    index = 0;
                // 顶点数据个数超过3个,即有效多边形
                if (this.pointList.length > 2) {
                    for (let i = 1; i < this.pointList.length; i++) {
                        let dis = SMathUtil.pointToLine(
                            new SPoint(event.x, event.y),
                            new SLine(this.pointList[i], this.pointList[i + 1])
                        );
                        // 是最后一个点,则使用最后一个点链接第一个点生成线
                        if (i + 1 == this.pointList.length) {
                            dis = SMathUtil.pointToLine(
                                new SPoint(event.x, event.y),
                                new SLine(this.pointList[i], this.pointList[0])
                            );
                        }
                        // 该点在多变形的吸附范围内
                        if (dis.MinDis < len.MinDis) {
                            len = dis;
                            index = i;
                        }
                    }
                }

                // 判断是否有点
                if (len.Point) {
                    // 点在了多边形的边上
                    if (len.MinDis <= this.scenceLen) {
                        this.pointList.splice(index + 1, 0, len.Point);
                        // 记录新增顶点操作记录压入堆栈
                        this.recordAction(SGraphPointListInsert, [
                            this.pointList,
                            len.Point,
                            index + 1
                        ]);
                    } else {
                        //没点在多边形边上也没点在多边形顶点上
                        super.onMouseDown(event);
                    }
                }
            } else {
                // 当捕捉到顶点后 ，记录当前点的xy坐标，用于undo、redo操作
                this.curPoint = new SPoint(
                    this.pointList[this.curIndex].x,
                    this.pointList[this.curIndex].y
                );
            }

            // 刷新视图
            this.update();
        }
    }

    /////////////////////
    // undo、redo相关操作
    /**
     * 记录相关动作并推入栈中
     *
     * @param SGraphCommand      相关命令类
     * @param any                对应传入参数
     */
    protected recordAction(SGraphCommand: any, any: any[]): void {
        // 记录相关命令并推入堆栈中
        const sgraphcommand = new SGraphCommand(this.scene, this, ...any);
        this.undoStack.push(sgraphcommand);
    }

    /**
     * 执行取消操作执行
     */
    undo(): void {
        // 处于正常态
        if (this.status == SItemStatus.Normal) {
            return;
        }
        this.undoStack.undo();
    }

    /**
     * 执行重做操作执行
     */
    redo(): void {
        // 处于正常态
        if (this.status == SItemStatus.Normal) {
            return;
        }

        this.undoStack.redo();
    }

    ///////////////////////////////
    // 以下为鼠标事件

    /**
     * 鼠标双击事件
     *
     * @param event    事件参数
     * @return  是否处理该事件
     */
    onDoubleClick(event: SMouseEvent): boolean {
        // 处于正常态
        if (SItemStatus.Normal == this.status) {
            this.status = SItemStatus.Edit;
            this.grabItem(this);
        } else if (SItemStatus.Edit == this.status) {
            // 处于编辑态
            this.status = SItemStatus.Normal;
            this.releaseItem();
        }

        this.update();
        return true;
    }

    /**
     * 键盘事件
     *
     * @param event    事件参数
     * @return  是否处理该事件
     */
    onKeyDown(event: KeyboardEvent): boolean {
        // 处于正常态
        if (this.status == SItemStatus.Normal) {
            return false;
        } else if (this.status == SItemStatus.Create) {
            // 处于创建态
            if (event.code == "Enter") {
                // 按下 enter 键
                // 当顶点大于二个时才又条件执行闭合操作并清空堆栈
                if (this.pointList.length > 2) {
                    this.status = SItemStatus.Normal;
                    //3 传递完成事件状态
                    this.$emit("finishCreated");
                    //1 grabItem 置为null
                    this.releaseItem();
                }
            }
        } else if (this.status == SItemStatus.Edit) {
            // 处于编辑态
            if (event.key == "Alt") {
                // 按下 alt 键
                this.isAlt = true;
            }
        }

        this.update();
        return true;
    }

    /**
     * 键盘键抬起事件
     *
     * @param event    事件参数
     */
    onKeyUp(event: KeyboardEvent): void {
        if (this.status == SItemStatus.Edit) {
            // 处于编辑态
            if (event.key == "Alt") {
                // 按下 alt 键
                this.isAlt = false;
            } else if (event.keyCode == SKeyCode.Delete) {
                // 按下 delete 键
                // 当多边形的顶点大于三个允许删除点
                if (this.pointList.length > 3) {
                    // 顶点个数超过 3 个可以删除
                    this.deletePoint(this.curIndex);
                }
            }
        }

        this.update();
    }

    /**
     * 鼠标按下事件
     *
     * @param	event      事件参数
     * @return	 是否处理该事件
     */
    onMouseDown(event: SMouseEvent): boolean {
        // 按下 shift 键
        if (event.shiftKey) {
            event = this.compare(event);
        }

        // 处于创建态
        if (this.status == SItemStatus.Create) {
            // 新增顶点
            let len = -1;
            // 轮廓线数据有效
            if (this.pointList.length) {
                len = SMathUtil.pointDistance(
                    event.x,
                    event.y,
                    this.pointList[0].x,
                    this.pointList[0].y
                );
            }

            // 顶点个数超过 2 个且距离第一个点的距离大于0且在多边形吸附范围内
            if (this.pointList.length > 2 && len > 0 && len < this.scenceLen) {
                this.status = SItemStatus.Normal;
                //3 传递完成事件状态
                this.$emit("finishCreated");
                //1 grabItem 置为null
                this.releaseItem();
            } else {
                // 否则
                this.insertPoint(event.x, event.y);
                // 记录新增顶点操作记录压入堆栈
                let pos = new SPoint(event.x, event.y);
                this.recordAction(SGraphPointListInsert, [this.pointList, pos]);
            }
        } else if (this.status == SItemStatus.Edit) {
            // 处于编辑态,对多边形数组做编辑操作
            this.editPolygonPoint(event);
        } else {
            // 否则
            // return super.onMouseDown(event);
            super.onMouseDown(event);
        }

        return true;
    }

    /**
     * 鼠标移入事件
     *
     * @param event   事件参数
     * @return 是否处理该事件
     */
    onMouseEnter(event: SMouseEvent): boolean {
        return true;
    }

    /**
     * 鼠标移出事件
     *
     * @param event    事件参数
     * @return 是否处理该事件
     */
    onMouseLeave(event: SMouseEvent): boolean {
        return true;
    }

    /**
     * 鼠标移动事件
     *
     * @param event    事件参数
     * @return 是否处理该事件
     */
    onMouseMove(event: SMouseEvent): boolean {
        // shift 键被按下
        if (event.shiftKey) {
            event = this.compare(event);
        }

        // 处于创建态
        if (this.status == SItemStatus.Create) {
            this.lastPoint = new SPoint();
            this.lastPoint.x = event.x;
            this.lastPoint.y = event.y;
            this.update();
        } else if (this.status == SItemStatus.Edit) {
            // 处于编辑态
            // 鼠标左键移动
            if (event.buttons == 1) {
                // 有选中的点
                if (-1 != this.curIndex) {
                    this.pointList[this.curIndex].x = event.x;
                    this.pointList[this.curIndex].y = event.y;
                }
            }
            this.update();
        } else {
            // 否则
            return super.onMouseMove(event);
        }

        return true;
    }

    /**
     * shift 垂直水平创建或编辑
     *
     * @param event   事件
     */
    compare(event: SMouseEvent): SMouseEvent {
        // 轮廓线有效
        if (this.pointList.length) {
            let last = new SPoint(event.x, event.y);
            // 处于创建态
            if (this.status == SItemStatus.Create) {
                last = this.pointList[this.pointList.length - 1];
            } else if (this.status == SItemStatus.Edit) {
                // 处于编辑态
                // 当前有选中的点
                if (this.curIndex > 0) {
                    last = this.pointList[this.curIndex - 1];
                }
            }

            const dx = Math.abs(event.x - last.x);
            const dy = Math.abs(event.y - last.y);
            // y 轴方向偏移量较大,则锁定 x 坐标
            if (dy > dx) {
                event.x = last.x;
            } else {
                // 否则,锁定 y 轴坐标
                event.y = last.y;
            }
        }

        return event;
    }

    /**
     * 鼠标抬起事件
     *
     * @param event    事件参数
     * @return 是否处理该事件
     */
    onMouseUp(event: SMouseEvent): boolean {
        // 处于编辑态
        if (this.status == SItemStatus.Edit) {
            // 当前有选中的点
            if (-1 != this.curIndex) {
                const pos = new SPoint(
                    this.pointList[this.curIndex].x,
                    this.pointList[this.curIndex].y
                );
                this.recordAction(SGraphPointListUpdate, [
                    this.pointList,
                    this.curPoint,
                    pos,
                    this.curIndex
                ]);
            }
        } else if (this.status == SItemStatus.Normal) {
            // 处于正常态
            return super.onMouseUp(event);
        }
        return true;
    }

    /**
     * 移动后处理所有坐标,并肩原点置为场景原点
     */
    moveToOrigin(): void {
        // 遍历顶点列表
        this.pointList = this.pointList.map(t => {
            t.x = t.x + this.x;
            t.y = t.y + this.y;
            return t;
        });

        this.x = 0;
        this.y = 0;
    }

    /**
     * 适配事件
     *
     * @param event     事件参数
     * @return 是否处理该事件
     */
    onResize(event: SMouseEvent): boolean {
        return true;
    }

    /**
     * 取消操作
     */
    cancelOperate(): void {
        // 当状态为展示状态
        if (this.status == SItemStatus.Create) {
            // 处于创建态
            this.parent = null;
        } else if (this.status == SItemStatus.Edit) {
            // 处于编辑状态
            this.status = SItemStatus.Normal;
        }

        this.update();
    }

    /**
     * Item 对象边界区域
     *
     * @return 边界区域
     */
    boundingRect(): SRect {
        // 轮廓线有效
        if (this.pointList.length) {
            this.minX = this.pointList[0].x;
            this.maxX = this.pointList[0].x;
            this.minY = this.pointList[0].y;
            this.maxY = this.pointList[0].y;
            // 遍历顶点列表
            this.pointList.forEach(it => {
                let x = it.x,
                    y = it.y;
                // 保存 x 最小值
                if (x < this.minX) {
                    this.minX = x;
                }

                // 保存 y 最小值
                if (y < this.minY) {
                    this.minY = y;
                }

                // 保存 x 最大值
                if (x > this.maxX) {
                    this.maxX = x;
                }

                // 保存 y 最大值
                if (y > this.maxY) {
                    this.maxY = y;
                }
            });
        }

        return new SRect(
            this.minX,
            this.minY,
            this.maxX - this.minX,
            this.maxY - this.minY
        );
    }

    /**
     * 判断点是否在区域内
     *
     * @param x     x 坐标
     * @param y     y 坐标
     * @return 是否在该区域内
     */
    contains(x: number, y: number): boolean {
        let arr = this.pointList;
        // 轮廓线数据无效或者该点不在多边形内部
        if (arr.length < 3 || !SPolygonUtil.pointIn(x, y, arr)) {
            return false;
        }

        return true;
    }

    /**
     * 返回对象储存的相关数据
     *
     * @return	对象参数
     */
    toData(): any {
        this.moveToOrigin();
        // 遍历顶点列表
        const Line = this.pointList.map(pos => {
            return {
                x: pos.x,
                y: pos.y
            };
        });

        this.data.style.outLine = Line;
        this.data.style.default.lineWidth = this.lineWidth;
        this.data.style.default.lineStyle = this.lineStyle;
        this.data.style.default.strokeColor = this.strokeColor.value;
        this.data.style.default.fillColor = this.fillColor.value;
        return this.data;
    }

    /**
     * 选择器放缩控制大小变化'
     *
     * @param old    旧的大小
     * @param newS   新的大小
     */
    resize(old: SSize, newS: SSize): void {
        const xs = newS.width / old.width;
        const ys = newS.height / old.height;
        // 遍历顶点列表,
        this.pointList = this.pointList.map(item => {
            item.x = item.x * xs;
            item.y = item.y * ys;
            return item;
        });
    }

    /**
     * Item 绘制操作
     *
     * @param painter    绘制对象
     */
    onDraw(painter: SPainter): void {
        this.scenceLen = painter.toPx(this.len);
        // 当状态为展示状态
        if (this.status == SItemStatus.Normal) {
            // 处于正常态
            this.drawShowPolygon(painter, this.pointList);
        } else if (this.status == SItemStatus.Create) {
            // 处于创建态
            this.drawCreatePolygon(painter, this.pointList);
        } else {
            // 处于编辑状态
            this.drawEditPolygon(painter, this.pointList);
        }
    }
}
