/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2021.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SGraphCommand } from "./SGraphCommand";
import { SUndoCommand } from "@persagy-web/base";
import { SGraphScene } from "../SGraphScene";
import { SGraphItem } from "../SGraphItem";
import { SPoint } from "@persagy-web/draw";

export default class SGraphGroupMoveCommand extends SGraphCommand {
    /** 命令名称 */
    readonly command: string;
    /** 执行命令的 item 的列表 */
    item: SGraphItem[];
    /** 移动前位置 */
    old: SPoint[];
    /** 移动后位置 */
    pos: SPoint[];

    /**
     * 构造函数
     *
     * @param scene     当前场景
     * @param itemList  item 对象列表
     * @param old       移动前位置
     * @param pos       移动后位置
     */
    constructor(
        scene: SGraphScene,
        itemList: SGraphItem[],
        old: SPoint[],
        pos: SPoint[]
    ) {
        super(scene);
        this.item = itemList;
        this.old = old;
        this.pos = pos;
        this.command = "SGraphGroupMoveCommand";
        this.desc = `移动对象列表=${JSON.stringify(itemList.map(t => t.id))}`;
    }

    /**
     * 合并命令
     *
     * @param command   指定命令
     * @return 是否已执行合并命令
     */
    mergeWith(command: SUndoCommand): boolean {
        return false;
    }

    /**
     * 重做
     */
    redo(): void {
        if (
            this.item.length > 0 &&
            this.item.length == this.old.length &&
            this.old.length == this.pos.length
        ) {
            this.item.forEach((t, i): void => {
                t.pos = new SPoint(this.pos[i]);
            });
            this.item[0].update();
        }
    }

    /**
     * 撤销
     */
    undo(): void {
        if (
            this.item.length > 0 &&
            this.item.length == this.old.length &&
            this.old.length == this.pos.length
        ) {
            this.item.forEach((t, i) => {
                t.pos = new SPoint(this.old[i]);
            });
            this.item[0].update();
        }
    }

    /**
     * 命令细节描述
     *
     * @return 将当前命令转为字符串
     */
    toString(): string {
        return `oldPos=${JSON.stringify(this.old)};\nnewPos=${JSON.stringify(
            this.pos
        )}`;
    }
}
